Package: textclassificationtutorial
Title: Reproducible Text Classification Workflows
Version: 0.1.2
Authors@R: c(
    person("Vladimer", "Kobayashi", role = c("aut", "cre"),
           email = "vladimer.kobayashi@gmail.com"),
    person("Stefan", "Mol", role = "aut"),
    person("Gabor", "Kismihok", role = "aut")
  )
Description: Dependency-light tools and tutorials for teaching reproducible
    text classification. The package covers HTML text extraction, sentence
    segmentation, text preprocessing, document-term matrices, TF-IDF,
    keyword extraction, cosine similarity, stratified cross-validation,
    classification metrics, and a multinomial Naive Bayes classifier. It 
    modernizes the code accompanying 
    Kobayashi, V. B., Berkers, H. A., Mol, S. T. Kismihok, G., and Den Hartog, D. N. (2017) <doi:10.1177/1094428117719322>
    The package replaces the original scripts in the paper.
License: Apache License (>= 2)
URL: https://github.com/vkobayashi/textclassificationtutorial
BugReports: https://github.com/vkobayashi/textclassificationtutorial/issues
Depends: R (>= 4.1.0)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), xml2
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-20 21:22:17 UTC; vladimerkobayashi
Author: Vladimer Kobayashi [aut, cre],
  Stefan Mol [aut],
  Gabor Kismihok [aut]
Maintainer: Vladimer Kobayashi <vladimer.kobayashi@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-26 19:30:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(predict,text_nb)
S3method(print,text_dtm)
S3method(print,text_nb)
export(classification_metrics)
export(cosine_similarity)
export(document_term_matrix)
export(extract_html_dir)
export(extract_html_text)
export(extract_keywords)
export(f_measure)
export(fit_naive_bayes)
export(preprocess_text)
export(read_label_file)
export(split_sentences)
export(stratified_folds)
export(tf_idf)

# textclassificationtutorial

`textclassificationtutorial` is an installable R package for learning and teaching
reproducible text classification. It modernizes the code accompanying:

> Kobayashi, V. B., Mol, S. T., Berkers, H. A., Kismihók, G., & Den Hartog,
> D. N. (2018). Text classification for organizational researchers: A
> tutorial. *Organizational Research Methods, 21*(3), 766–799.

The package turns the original sequence-dependent scripts into documented,
testable functions.

## Installation

Install the released version from CRAN, or obtain the development source from
the [GitHub repository](https://github.com/vkobayashi/textclassificationtutorial).
No package function, example, test, or vignette installs dependencies.


## Quick start

```r
library(textclassificationtutorial)

documents <- c(
  "Analyze customer data and build statistical models.",
  "Create dashboards and communicate analytical findings.",
  "Provide nursing care and support patients.",
  "Coordinate treatment with physicians and nurses."
)

labels <- c("data", "data", "care", "care")

clean <- preprocess_text(
  documents,
  stopwords = c("and", "with"),
  min_token_length = 2
)

dtm <- document_term_matrix(clean)
tf_idf(dtm)
extract_keywords(dtm, n = 2)

model <- fit_naive_bayes(dtm, labels)
predictions <- predict(model, dtm)
classification_metrics(labels, predictions, positive = "data")
```

## What the package covers

- HTML extraction from a file, string, or directory
- Lightweight sentence segmentation
- Transparent text normalization and stopword removal
- Document-term and TF-IDF matrices
- Keyword extraction and cosine similarity
- Stratified repeated cross-validation folds
- Binary classification evaluation
- Multinomial Naive Bayes without a heavy modelling dependency

Read the tutorials with:

```r
vignette("getting-started", package = "textclassificationtutorial")
vignette("model-evaluation", package = "textclassificationtutorial")
```

## Design principles

Package functions never install dependencies, change global warning settings,
open browsers, delete output directories, or depend on objects produced by a
previous script. File-writing decisions remain with the user.

The lightweight sentence splitter and classifier are deliberately transparent
for teaching. For production NLP, use language-specific tokenizers, carefully
validated preprocessing, and models appropriate to the research design.

## Development

Development dependencies must be installed explicitly by the developer before
running documentation, tests, or package checks. The package never installs
missing packages automatically.

#' Compute cosine similarity
#'
#' @param x Numeric matrix whose rows are observations.
#' @param y Optional numeric matrix with the same columns as `x`. When omitted,
#'   computes all pairwise similarities among rows of `x`.
#'
#' @return A numeric similarity matrix. Similarities involving zero vectors
#'   are returned as `NA`.
#' @export
#'
#' @examples
#' x <- rbind(a = c(1, 1, 0), b = c(1, 0, 0), c = c(0, 0, 1))
#' cosine_similarity(x)
cosine_similarity <- function(x, y = NULL) {
  if (!is.matrix(x) || !is.numeric(x)) {
    stop("`x` must be a numeric matrix.", call. = FALSE)
  }
  if (is.null(y)) y <- x
  if (!is.matrix(y) || !is.numeric(y) || ncol(y) != ncol(x)) {
    stop("`y` must be a numeric matrix with the same columns as `x`.",
         call. = FALSE)
  }
  denominator <- outer(sqrt(rowSums(x^2)), sqrt(rowSums(y^2)))
  result <- tcrossprod(x, y) / denominator
  result[denominator == 0] <- NA_real_
  dimnames(result) <- list(rownames(x), rownames(y))
  result
}

test_that("cosine similarity has known values", {
  x <- rbind(a = c(1, 0), b = c(1, 1), c = c(0, 0))
  result <- cosine_similarity(x)
  expect_equal(result["a", "a"], 1)
  expect_equal(result["a", "b"], 1 / sqrt(2))
  expect_true(is.na(result["a", "c"]))
})

test_that("cosine similarity validates dimensions", {
  expect_error(cosine_similarity(data.frame(a = 1)), "matrix")
  expect_error(cosine_similarity(matrix(1, 1, 2), matrix(1, 1, 3)),
               "same columns")
})

#' Extract readable text from HTML
#'
#' Extracts text from an HTML file or character string. When the suggested
#' `xml2` package is installed, simple CSS selectors (`tag`, `.class`, `#id`,
#' or `tag.class`) or XPath can target a specific part of the page. A
#' dependency-free fallback strips markup from the full document.
#'
#' @param x Path to an HTML file or a length-one HTML character string.
#' @param selector Optional CSS selector.
#' @param xpath Optional XPath expression. Supply at most one of `selector`
#'   and `xpath`.
#' @param collapse Character used to join matched nodes.
#' @param trim Logical; normalize whitespace and trim the result?
#'
#' @return A length-one character vector containing extracted text.
#' @export
#'
#' @examples
#' html <- "<html><body><h1>Analyst</h1><p>Analyze data.</p></body></html>"
#' extract_html_text(html)
extract_html_text <- function(x, selector = NULL, xpath = NULL,
                              collapse = "\n", trim = TRUE) {
  if (!is.character(x) || length(x) != 1L || is.na(x)) {
    stop("`x` must be one HTML string or file path.", call. = FALSE)
  }
  if (!is.null(selector) && !is.null(xpath)) {
    stop("Supply only one of `selector` and `xpath`.", call. = FALSE)
  }
  is_file <- file.exists(x)
  html <- if (is_file) {
    paste(readLines(x, warn = FALSE, encoding = "UTF-8"), collapse = "\n")
  } else {
    x
  }

  if (requireNamespace("xml2", quietly = TRUE)) {
    document <- if (is_file) xml2::read_html(x) else xml2::read_html(html)
    unwanted <- xml2::xml_find_all(document, ".//script|.//style")
    if (length(unwanted)) xml2::xml_remove(unwanted)
    nodes <- if (!is.null(selector)) {
      xml2::xml_find_all(document, css_to_xpath(selector))
    } else if (!is.null(xpath)) {
      xml2::xml_find_all(document, xpath)
    } else {
      xml2::xml_find_all(document, ".//body")
    }
    if (!length(nodes)) return(character())
    text <- paste(xml2::xml_text(nodes, trim = trim), collapse = collapse)
  } else {
    if (!is.null(selector) || !is.null(xpath)) {
      stop("Install `xml2` to use CSS or XPath selection.", call. = FALSE)
    }
    html <- gsub("(?is)<(script|style)[^>]*>.*?</\\1>", " ", html,
                 perl = TRUE)
    html <- gsub("(?i)<br\\s*/?>|</p>|</li>|</h[1-6]>", "\n", html,
                 perl = TRUE)
    text <- gsub("(?s)<[^>]+>", " ", html, perl = TRUE)
    text <- decode_html_entities(text)
  }
  if (trim) normalize_text_whitespace(text) else text
}

#' Extract text from a directory of HTML files
#'
#' @param path Directory containing HTML files.
#' @param pattern File-name regular expression.
#' @param recursive Logical; search recursively?
#' @param ... Passed to [extract_html_text()].
#'
#' @return A data frame with `document_id`, `path`, and `text`.
#' @export
extract_html_dir <- function(path, pattern = "\\.html?$", recursive = FALSE,
                             ...) {
  if (!dir.exists(path)) stop("Directory does not exist: ", path,
                              call. = FALSE)
  files <- list.files(path, pattern = pattern, full.names = TRUE,
                      recursive = recursive, ignore.case = TRUE)
  files <- sort(files)
  if (!length(files)) {
    return(data.frame(document_id = character(), path = character(),
                      text = character()))
  }
  data.frame(
    document_id = tools::file_path_sans_ext(basename(files)),
    path = normalizePath(files, winslash = "/", mustWork = TRUE),
    text = vapply(files, extract_html_text, character(1), ...),
    stringsAsFactors = FALSE
  )
}

decode_html_entities <- function(x) {
  replacements <- c(
    "&nbsp;" = " ", "&amp;" = "&", "&lt;" = "<", "&gt;" = ">",
    "&quot;" = "\"", "&#39;" = "'"
  )
  for (entity in names(replacements)) {
    x <- gsub(entity, replacements[[entity]], x, fixed = TRUE)
  }
  numeric_entity <- "&#([0-9]+);"
  while (grepl(numeric_entity, x, perl = TRUE)) {
    value <- as.integer(sub(paste0(".*?", numeric_entity, ".*"), "\\1", x,
                            perl = TRUE))
    x <- sub(numeric_entity, intToUtf8(value), x, perl = TRUE)
  }
  x
}

normalize_text_whitespace <- function(x) {
  x <- gsub("[\t\r ]+", " ", x)
  x <- gsub(" *\\n+ *", "\n", x)
  trimws(x)
}

css_to_xpath <- function(selector) {
  if (!is.character(selector) || length(selector) != 1L ||
      is.na(selector) || !nzchar(selector)) {
    stop("`selector` must be one non-empty CSS selector.", call. = FALSE)
  }
  selector <- trimws(selector)
  if (grepl("^#[A-Za-z][A-Za-z0-9_-]*$", selector)) {
    return(paste0(".//*[@id='", substring(selector, 2), "']"))
  }
  if (grepl("^\\.[A-Za-z][A-Za-z0-9_-]*$", selector)) {
    class <- substring(selector, 2)
    return(paste0(
      ".//*[contains(concat(' ', normalize-space(@class), ' '), ' ",
      class, " ')]"
    ))
  }
  if (grepl("^[A-Za-z][A-Za-z0-9_-]*\\.[A-Za-z][A-Za-z0-9_-]*$",
            selector)) {
    parts <- strsplit(selector, ".", fixed = TRUE)[[1]]
    return(paste0(
      ".//", parts[1],
      "[contains(concat(' ', normalize-space(@class), ' '), ' ",
      parts[2], " ')]"
    ))
  }
  if (grepl("^[A-Za-z][A-Za-z0-9_-]*$", selector)) {
    return(paste0(".//", selector))
  }
  stop(
    "Supported CSS selectors are tag, .class, #id, and tag.class. ",
    "Use `xpath` for complex selections.",
    call. = FALSE
  )
}

test_that("HTML text is extracted without markup", {
  html <- paste0(
    "<html><head><style>hidden</style></head><body>",
    "<h1>Data Scientist</h1><p>Analyze &amp; model data.</p>",
    "<script>ignored()</script></body></html>"
  )
  result <- extract_html_text(html)
  expect_match(result, "Data Scientist")
  expect_match(result, "Analyze & model data")
  expect_false(grepl("<[^>]+>", result))
  expect_false(grepl("ignored", result))
})

test_that("directory extraction is ordered and identified", {
  path <- tempfile(pattern = "html-files-", tmpdir = tempdir())
  dir.create(path, recursive=TRUE)
  on.exit(unlink(path, recursive = TRUE, force = TRUE), add = TRUE)
  writeLines("<body>Second</body>", con=file.path(path, "b.html"),useBytes = TRUE)
  writeLines("<body>First</body>", file.path(path, "a.html"),useBytes = TRUE)
  result <- extract_html_dir(path)
  expect_equal(result$document_id, c("a", "b"))
  expect_equal(result$text, c("First", "Second"))
})

test_that("selectors are validated", {
  expect_error(
    extract_html_text("<p>x</p>", selector = "p", xpath = "//p"),
    "only one"
  )
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## -----------------------------------------------------------------------------
library(textclassificationtutorial)

## -----------------------------------------------------------------------------
html_file <- system.file(
  "extdata", "sample_nursing_vacancy.html",
  package = "textclassificationtutorial"
)
vacancy_text <- extract_html_text(html_file)
substr(vacancy_text, 1, 200)

## ----eval = FALSE-------------------------------------------------------------
# pages <- extract_html_dir("inst/extdata/vacancypages")

## ----eval = FALSE-------------------------------------------------------------
# extract_html_text(html_file, selector = "div.content")
# extract_html_text(html_file, xpath = "//div[@class='content']")

## -----------------------------------------------------------------------------
sentences <- split_sentences(vacancy_text)
head(sentences)

## -----------------------------------------------------------------------------
german_stopwords <- c(
  "der", "die", "das", "den", "dem", "des", "und", "oder", "mit",
  "für", "von", "zu", "im", "in", "auf", "ein", "eine"
)

clean <- preprocess_text(
  sentences,
  lowercase = TRUE,
  remove_punctuation = TRUE,
  remove_numbers = TRUE,
  stopwords = german_stopwords,
  min_token_length = 2
)
clean <- clean[nzchar(clean)]
head(clean)

## -----------------------------------------------------------------------------
dtm <- document_term_matrix(
  clean,
  min_doc_freq = 2,
  max_doc_prop = 0.95
)
dtm

## -----------------------------------------------------------------------------
weighted <- tf_idf(dtm)
keywords <- extract_keywords(dtm, n = 3)
head(keywords, 12)

## -----------------------------------------------------------------------------
similarity <- cosine_similarity(weighted)
round(similarity[1:min(5, nrow(similarity)),
                 1:min(5, ncol(similarity))], 2)

## -----------------------------------------------------------------------------
training_text <- c(
  "analyze data statistical model",
  "build predictive model data",
  "create dashboard analyze metrics",
  "provide nursing care patient",
  "support patient clinical care",
  "coordinate nurse patient treatment"
)
training_labels <- c("data", "data", "data", "care", "care", "care")

training_dtm <- document_term_matrix(training_text)
model <- fit_naive_bayes(training_dtm, training_labels, laplace = 1)
model

predicted <- predict(model, training_dtm)
classification_metrics(training_labels, predicted, positive = "data")

#' Construct a document-term matrix
#'
#' @param text Character vector containing one preprocessed document per item.
#' @param document_ids Optional unique document identifiers.
#' @param binary Logical; store term presence instead of term frequency?
#' @param min_doc_freq Minimum number of documents in which a term must occur.
#' @param max_doc_prop Maximum proportion of documents in which a term may
#'   occur.
#'
#' @return A numeric matrix with class `text_dtm`.
#' @export
#'
#' @examples
#' docs <- preprocess_text(c("data science", "data analysis", "science"))
#' document_term_matrix(docs)
document_term_matrix <- function(text, document_ids = NULL, binary = FALSE,
                                 min_doc_freq = 1L, max_doc_prop = 1) {
  if (!is.character(text)) stop("`text` must be character.", call. = FALSE)
  n <- length(text)
  if (!n) stop("`text` must contain at least one document.", call. = FALSE)
  if (is.null(document_ids)) document_ids <- paste0("doc", seq_len(n))
  if (length(document_ids) != n || anyNA(document_ids) ||
      anyDuplicated(document_ids)) {
    stop("`document_ids` must be unique and match the number of documents.",
         call. = FALSE)
  }
  if (length(min_doc_freq) != 1L || is.na(min_doc_freq) ||
      min_doc_freq < 1) {
    stop("`min_doc_freq` must be at least 1.", call. = FALSE)
  }
  if (length(max_doc_prop) != 1L || is.na(max_doc_prop) ||
      max_doc_prop <= 0 || max_doc_prop > 1) {
    stop("`max_doc_prop` must be in (0, 1].", call. = FALSE)
  }

  tokens <- strsplit(trimws(text), "\\s+", perl = TRUE)
  tokens <- lapply(tokens, function(x) x[nzchar(x)])
  vocabulary <- sort(unique(unlist(tokens, use.names = FALSE)))
  matrix <- matrix(0, nrow = n, ncol = length(vocabulary),
                   dimnames = list(as.character(document_ids), vocabulary))
  if (length(vocabulary)) {
    for (i in seq_along(tokens)) {
      counts <- table(tokens[[i]])
      matrix[i, names(counts)] <- as.numeric(counts)
    }
    doc_frequency <- colSums(matrix > 0)
    keep <- doc_frequency >= min_doc_freq &
      doc_frequency / n <= max_doc_prop
    matrix <- matrix[, keep, drop = FALSE]
  }
  if (binary) matrix[] <- as.numeric(matrix > 0)
  class(matrix) <- c("text_dtm", "matrix", "array")
  attr(matrix, "binary") <- binary
  matrix
}

#' @export
print.text_dtm <- function(x, ...) {
  cat("<text_dtm> ", nrow(x), " documents x ", ncol(x), " terms\n", sep = "")
  print(unclass(x), ...)
  invisible(x)
}

#' Calculate TF-IDF weights
#'
#' @param dtm Numeric document-term matrix.
#' @param normalize Term-frequency normalization: document length, maximum
#'   frequency, or none.
#' @param smooth_idf Logical; use smoothed inverse document frequency?
#'
#' @return A numeric matrix of TF-IDF weights.
#' @export
tf_idf <- function(dtm, normalize = c("length", "max", "none"),
                   smooth_idf = TRUE) {
  normalize <- match.arg(normalize)
  validate_dtm(dtm)
  tf <- unclass(as.matrix(dtm))
  if (normalize == "length") {
    denominator <- rowSums(tf)
    denominator[denominator == 0] <- 1
    tf <- tf / denominator
  } else if (normalize == "max") {
    denominator <- apply(tf, 1, max)
    denominator[denominator == 0] <- 1
    tf <- tf / denominator
  }
  document_frequency <- colSums(dtm > 0)
  idf <- if (smooth_idf) {
    log((1 + nrow(dtm)) / (1 + document_frequency)) + 1
  } else {
    result <- log(nrow(dtm) / document_frequency)
    result[!is.finite(result)] <- 0
    result
  }
  sweep(tf, 2, idf, `*`)
}

#' Extract top TF-IDF keywords
#'
#' @param x A document-term matrix or TF-IDF matrix.
#' @param n Number of keywords per document.
#' @param already_tfidf Logical; is `x` already weighted?
#'
#' @return A data frame with document, rank, term, and weight.
#' @export
extract_keywords <- function(x, n = 1L, already_tfidf = FALSE) {
  validate_dtm(x)
  if (length(n) != 1L || is.na(n) || n < 1) {
    stop("`n` must be at least 1.", call. = FALSE)
  }
  weights <- if (already_tfidf) as.matrix(x) else tf_idf(x)
  output <- lapply(seq_len(nrow(weights)), function(i) {
    values <- weights[i, ]
    order_index <- order(values, decreasing = TRUE)
    order_index <- order_index[values[order_index] > 0]
    order_index <- utils::head(order_index, n)
    if (!length(order_index)) {
      return(data.frame(
        document = character(), rank = integer(), term = character(),
        weight = numeric(), stringsAsFactors = FALSE
      ))
    }
    data.frame(
      document = rownames(weights)[i] %||% as.character(i),
      rank = seq_along(order_index),
      term = colnames(weights)[order_index],
      weight = unname(values[order_index]),
      stringsAsFactors = FALSE
    )
  })
  do.call(rbind, output)
}

validate_dtm <- function(x) {
  if (!is.matrix(x) || !is.numeric(x) || is.null(colnames(x))) {
    stop("`x` must be a numeric matrix with term column names.",
         call. = FALSE)
  }
  invisible(TRUE)
}

`%||%` <- function(x, y) if (is.null(x)) y else x

test_that("DTM contains expected counts", {
  dtm <- document_term_matrix(
    c("data science data", "science model"),
    document_ids = c("a", "b")
  )
  expect_s3_class(dtm, "text_dtm")
  expect_equal(unname(dtm["a", "data"]), 2)
  expect_equal(unname(dtm["b", "model"]), 1)
  expect_equal(rownames(dtm), c("a", "b"))
})

test_that("document frequency filters work", {
  dtm <- document_term_matrix(c("a b", "a c", "a d"), min_doc_freq = 2)
  expect_equal(colnames(dtm), "a")
  dtm2 <- document_term_matrix(c("a b", "a c", "a d"), max_doc_prop = 0.9)
  expect_false("a" %in% colnames(dtm2))
})

test_that("TF-IDF and keyword extraction are well formed", {
  dtm <- document_term_matrix(c("data data model", "care patient"))
  weighted <- tf_idf(dtm)
  expect_equal(dim(weighted), dim(dtm))
  expect_true(all(is.finite(weighted)))
  keywords <- extract_keywords(dtm, n = 2)
  expect_named(keywords, c("document", "rank", "term", "weight"))
  expect_true(all(keywords$weight > 0))
})

test_that("invalid DTM inputs fail clearly", {
  expect_error(document_term_matrix(character()), "at least one")
  expect_error(document_term_matrix(c("a", "b"), document_ids = c("x", "x")),
               "unique")
})

#' F-measure
#'
#' @param precision Numeric precision.
#' @param recall Numeric recall.
#' @param beta Relative weight assigned to recall.
#'
#' @return Numeric F-measure.
#' @export
f_measure <- function(precision, recall, beta = 1) {
  if (any(c(precision, recall) < 0, na.rm = TRUE) ||
      any(c(precision, recall) > 1, na.rm = TRUE)) {
    stop("`precision` and `recall` must be between 0 and 1.",
         call. = FALSE)
  }
  if (length(beta) != 1L || is.na(beta) || beta <= 0) {
    stop("`beta` must be positive.", call. = FALSE)
  }
  denominator <- beta^2 * precision + recall
  result <- (1 + beta^2) * precision * recall / denominator
  result[!is.na(denominator) & denominator == 0] <- 0
  result
}

#' Calculate binary classification metrics
#'
#' @param truth Vector of true classes.
#' @param estimate Vector of predicted classes.
#' @param positive Value identifying the positive class.
#'
#' @return A one-row data frame containing confusion counts, accuracy,
#'   balanced accuracy, precision, recall, specificity, and F1.
#' @export
#'
#' @examples
#' classification_metrics(c("task", "task", "other"), c("task", "other", "other"),
#'                        positive = "task")
classification_metrics <- function(truth, estimate, positive) {
  if (length(truth) != length(estimate) || !length(truth)) {
    stop("`truth` and `estimate` must have the same nonzero length.",
         call. = FALSE)
  }
  if (length(positive) != 1L || is.na(positive)) {
    stop("`positive` must identify one class.", call. = FALSE)
  }
  complete <- !is.na(truth) & !is.na(estimate)
  truth <- truth[complete]
  estimate <- estimate[complete]
  actual_positive <- truth == positive
  predicted_positive <- estimate == positive
  tp <- sum(actual_positive & predicted_positive)
  fn <- sum(actual_positive & !predicted_positive)
  fp <- sum(!actual_positive & predicted_positive)
  tn <- sum(!actual_positive & !predicted_positive)
  safe_ratio <- function(a, b) if (b == 0) NA_real_ else a / b
  precision <- safe_ratio(tp, tp + fp)
  recall <- safe_ratio(tp, tp + fn)
  specificity <- safe_ratio(tn, tn + fp)
  data.frame(
    n = length(truth), true_positive = tp, false_positive = fp,
    true_negative = tn, false_negative = fn,
    accuracy = safe_ratio(tp + tn, length(truth)),
    balanced_accuracy = mean(c(recall, specificity), na.rm = TRUE),
    precision = precision, recall = recall, specificity = specificity,
    f1 = if (is.na(precision) || is.na(recall)) NA_real_ else
      f_measure(precision, recall)
  )
}

#' Create stratified cross-validation folds
#'
#' @param y Class labels.
#' @param k Number of folds.
#' @param repeats Number of repeated fold sets.
#' @param seed Optional random seed. The caller's random-number state is
#'   restored.
#'
#' @return A list of integer test-set indices with class `text_folds`.
#' @export
stratified_folds <- function(y, k = 5L, repeats = 1L, seed = NULL) {
  if (length(y) < 2L || anyNA(y)) {
    stop("`y` must contain at least two non-missing labels.", call. = FALSE)
  }
  counts <- table(y)
  if (length(counts) < 2L) stop("`y` must contain at least two classes.",
                                call. = FALSE)
  if (length(k) != 1L || is.na(k) || k < 2 || k > min(counts) ||
      k != as.integer(k)) {
    stop("`k` must be between 2 and the smallest class size.",
         call. = FALSE)
  }
  if (length(repeats) != 1L || is.na(repeats) || repeats < 1 ||
      repeats != as.integer(repeats)) {
    stop("`repeats` must be at least 1.", call. = FALSE)
  }
  k <- as.integer(k)
  repeats <- as.integer(repeats)
  with_preserved_seed(seed, {
    folds <- vector("list", k * repeats)
    position <- 1L
    for (repeat_id in seq_len(repeats)) {
      assignments <- integer(length(y))
      labels <- as.character(y)
      for (class in unique(labels)) {
        index <- which(labels == class)
        assignments[index] <- sample(rep(seq_len(k), length.out = length(index)))
      }
      for (fold_id in seq_len(k)) {
        folds[[position]] <- which(assignments == fold_id)
        position <- position + 1L
      }
    }
    names(folds) <- sprintf("Repeat%02d_Fold%02d",
                            rep(seq_len(repeats), each = k),
                            rep(seq_len(k), repeats))
    structure(folds, class = c("text_folds", "list"), k = k,
              repeats = repeats)
  })
}

with_preserved_seed <- function(seed, code) {
  if (is.null(seed)) return(force(code))
  existed <- exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
  if (existed) old_seed <- get(".Random.seed", envir = .GlobalEnv)
  on.exit({
    if (existed) {
      assign(".Random.seed", old_seed, envir = .GlobalEnv)
    } else if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      rm(".Random.seed", envir = .GlobalEnv)
    }
  }, add = TRUE)
  set.seed(seed)
  force(code)
}

test_that("classification metrics use correct confusion counts", {
  truth <- c("yes", "yes", "no", "no")
  estimate <- c("yes", "no", "yes", "no")
  result <- classification_metrics(truth, estimate, positive = "yes")
  expect_equal(result$true_positive, 1)
  expect_equal(result$false_positive, 1)
  expect_equal(result$true_negative, 1)
  expect_equal(result$false_negative, 1)
  expect_equal(result$balanced_accuracy, 0.5)
  expect_equal(result$f1, 0.5)
})

test_that("F-measure handles zero precision and recall", {
  expect_equal(f_measure(0, 0), 0)
  expect_equal(f_measure(1, 1), 1)
  expect_error(f_measure(1.2, 1), "between 0 and 1")
})

test_that("folds are stratified, complete, and reproducible", {
  y <- rep(c("a", "b"), each = 6)
  first <- stratified_folds(y, k = 3, repeats = 2, seed = 42)
  second <- stratified_folds(y, k = 3, repeats = 2, seed = 42)
  expect_equal(first, second)
  expect_length(first, 6)
  for (fold in first) expect_setequal(unique(y[fold]), c("a", "b"))
  expect_setequal(unlist(first[1:3]), seq_along(y))
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")

## -----------------------------------------------------------------------------
library(textclassificationtutorial)

text <- c(
  "analyze customer data", "build statistical models",
  "create predictive analytics", "report business metrics",
  "provide patient care", "support clinical treatment",
  "coordinate nursing care", "assist hospital patients",
  "analyze experimental results", "develop data dashboard",
  "monitor patient health", "coordinate clinical team"
)
label <- rep(c("data", "care"), each = 4)
label <- c(label, "data", "data", "care", "care")

## -----------------------------------------------------------------------------
folds <- stratified_folds(label, k = 3, seed = 2026)
folds

## -----------------------------------------------------------------------------
fold_results <- lapply(folds, function(test_index) {
  train_index <- setdiff(seq_along(text), test_index)

  train_clean <- preprocess_text(text[train_index])
  test_clean <- preprocess_text(text[test_index])

  train_dtm <- document_term_matrix(train_clean)
  test_dtm <- document_term_matrix(test_clean)

  model <- fit_naive_bayes(train_dtm, label[train_index])
  estimate <- predict(model, test_dtm)

  classification_metrics(
    truth = label[test_index],
    estimate = estimate,
    positive = "data"
  )
})

results <- do.call(rbind, fold_results)
results
colMeans(results[c(
  "accuracy", "balanced_accuracy", "precision", "recall", "f1"
)], na.rm = TRUE)

#' Fit a multinomial Naive Bayes text classifier
#'
#' @param x Nonnegative numeric document-term matrix.
#' @param y Class labels with one value per row of `x`.
#' @param laplace Nonnegative additive smoothing parameter.
#' @param prior Optional named class probabilities.
#'
#' @return An object of class `text_nb`.
#' @export
#'
#' @examples
#' x <- rbind(c(3, 0), c(2, 0), c(0, 3), c(0, 2))
#' colnames(x) <- c("analysis", "care")
#' model <- fit_naive_bayes(x, c("data", "data", "health", "health"))
#' predict(model, x)
fit_naive_bayes <- function(x, y, laplace = 1, prior = NULL) {
  validate_dtm(x)
  if (nrow(x) != length(y) || anyNA(y)) {
    stop("`y` must contain one non-missing label per document.",
         call. = FALSE)
  }
  if (any(x < 0) || any(!is.finite(x))) {
    stop("`x` must contain finite nonnegative values.", call. = FALSE)
  }
  if (length(laplace) != 1L || is.na(laplace) || laplace < 0) {
    stop("`laplace` must be nonnegative.", call. = FALSE)
  }
  classes <- unique(as.character(y))
  if (length(classes) < 2L) stop("At least two classes are required.",
                                 call. = FALSE)
  y <- factor(y, levels = classes)
  if (is.null(prior)) {
    class_prior <- prop.table(table(y))
  } else {
    if (is.null(names(prior)) || !setequal(names(prior), classes) ||
        any(prior < 0) || abs(sum(prior) - 1) > sqrt(.Machine$double.eps)) {
      stop("`prior` must be named probabilities summing to 1 for every class.",
           call. = FALSE)
    }
    class_prior <- prior[classes]
  }
  term_counts <- vapply(
    classes,
    function(class) colSums(x[y == class, , drop = FALSE]),
    numeric(ncol(x))
  )
  term_probability <- sweep(
    term_counts + laplace,
    2,
    colSums(term_counts) + laplace * nrow(term_counts),
    `/`
  )
  structure(
    list(
      classes = classes,
      terms = colnames(x),
      prior = as.numeric(class_prior[classes]),
      term_probability = term_probability,
      laplace = laplace
    ),
    class = "text_nb"
  )
}

#' @export
predict.text_nb <- function(object, newdata, type = c("class", "prob"), ...) {
  type <- match.arg(type)
  validate_dtm(newdata)
  missing_terms <- setdiff(object$terms, colnames(newdata))
  if (length(missing_terms)) {
    padding <- matrix(0, nrow(newdata), length(missing_terms),
                      dimnames = list(rownames(newdata), missing_terms))
    newdata <- cbind(newdata, padding)
  }
  newdata <- newdata[, object$terms, drop = FALSE]
  log_scores <- newdata %*% log(object$term_probability)
  log_scores <- sweep(log_scores, 2, log(object$prior), `+`)
  row_max <- apply(log_scores, 1, max)
  probabilities <- exp(log_scores - row_max)
  probabilities <- probabilities / rowSums(probabilities)
  colnames(probabilities) <- object$classes
  rownames(probabilities) <- rownames(newdata)
  if (type == "prob") return(probabilities)
  factor(object$classes[max.col(probabilities, ties.method = "first")],
         levels = object$classes)
}

#' @export
print.text_nb <- function(x, ...) {
  cat("<text_nb> Multinomial Naive Bayes\n")
  cat("Classes: ", paste(x$classes, collapse = ", "), "\n", sep = "")
  cat("Terms: ", length(x$terms), "\n", sep = "")
  invisible(x)
}

training_dtm <- structure(
  rbind(
    c(3, 0), c(2, 0), c(0, 3), c(0, 2)
  ),
  dimnames = list(NULL, c("analysis", "care"))
)
training_labels <- c("data", "data", "health", "health")

test_that("Naive Bayes learns and predicts separable documents", {
  model <- fit_naive_bayes(training_dtm, training_labels)
  expect_s3_class(model, "text_nb")
  predictions <- predict(model, training_dtm)
  expect_equal(as.character(predictions), training_labels)
  probabilities <- predict(model, training_dtm, type = "prob")
  expect_equal(rowSums(probabilities), rep(1, 4), tolerance = 1e-12)
})

test_that("prediction handles absent and extra terms", {
  model <- fit_naive_bayes(training_dtm, training_labels)
  newdata <- matrix(c(2, 99), nrow = 1,
                    dimnames = list("new", c("analysis", "extra")))
  prediction <- predict(model, newdata)
  expect_equal(as.character(prediction), "data")
})

test_that("Naive Bayes validates model inputs", {
  expect_error(fit_naive_bayes(training_dtm, training_labels[1:3]),
               "one non-missing")
  expect_error(fit_naive_bayes(training_dtm, rep("data", 4)),
               "two classes")
  expect_error(fit_naive_bayes(training_dtm, training_labels, laplace = -1),
               "nonnegative")
})

#' Split text into sentences
#'
#' A lightweight sentence segmenter suitable for tutorials and clean prose.
#' It splits at terminal punctuation followed by whitespace, and optionally at
#' line breaks and vertical bars. For production multilingual segmentation,
#' use a dedicated NLP tokenizer.
#'
#' @param text Character vector.
#' @param split_lines Logical; treat line breaks and `|` as boundaries?
#' @param keep_punctuation Logical; retain terminal punctuation?
#' @param drop_empty Logical; remove empty results?
#'
#' @return A character vector of sentences.
#' @export
#'
#' @examples
#' split_sentences("Analyze data. Present results! Work with teams?")
split_sentences <- function(text, split_lines = TRUE, keep_punctuation = TRUE,
                            drop_empty = TRUE) {
  if (!is.character(text)) stop("`text` must be character.", call. = FALSE)
  text[is.na(text)] <- ""
  boundary <- if (keep_punctuation) {
    "(?<=[.!?])\\s+"
  } else {
    "[.!?]+\\s*"
  }
  if (split_lines) {
    text <- gsub("[\r\n|]+", "\n", text)
    boundary <- paste0(boundary, "|\\n+")
  }
  pieces <- unlist(strsplit(text, boundary, perl = TRUE), use.names = FALSE)
  pieces <- trimws(pieces)
  if (drop_empty) pieces[nzchar(pieces)] else pieces
}

#' Normalize text for document-term analysis
#'
#' @param text Character vector.
#' @param lowercase Logical; convert text to lowercase?
#' @param remove_punctuation Logical; replace punctuation with spaces?
#' @param remove_numbers Logical; replace digits with spaces?
#' @param stopwords Optional character vector of words to remove.
#' @param min_token_length Minimum number of characters per token.
#'
#' @return A character vector of normalized documents.
#' @export
#'
#' @examples
#' preprocess_text(
#'   c("Analyze the data!", "Present 2 reports."),
#'   stopwords = c("the")
#' )
preprocess_text <- function(text, lowercase = TRUE, remove_punctuation = TRUE,
                            remove_numbers = TRUE, stopwords = character(),
                            min_token_length = 1L) {
  if (!is.character(text)) stop("`text` must be character.", call. = FALSE)
  if (!is.character(stopwords)) stop("`stopwords` must be character.",
                                     call. = FALSE)
  if (length(min_token_length) != 1L || is.na(min_token_length) ||
      min_token_length < 1) {
    stop("`min_token_length` must be at least 1.", call. = FALSE)
  }
  text[is.na(text)] <- ""
  if (lowercase) {
    text <- tolower(text)
    stopwords <- tolower(stopwords)
  }
  if (remove_punctuation) text <- gsub("[[:punct:]]+", " ", text)
  if (remove_numbers) text <- gsub("[[:digit:]]+", " ", text)
  text <- gsub("[[:space:]]+", " ", text)
  text <- trimws(text)

  clean_one <- function(document) {
    tokens <- strsplit(document, "\\s+", perl = TRUE)[[1]]
    tokens <- tokens[nzchar(tokens)]
    tokens <- tokens[nchar(tokens) >= min_token_length]
    tokens <- tokens[!tokens %in% stopwords]
    paste(tokens, collapse = " ")
  }
  vapply(text, clean_one, character(1), USE.NAMES = FALSE)
}

#' Read a one-label-per-line file
#'
#' @param path Path to a text file.
#' @param type Return labels as character, integer, numeric, or factor.
#'
#' @return A vector of labels.
#' @export
read_label_file <- function(path,
                            type = c("character", "integer", "numeric",
                                     "factor")) {
  type <- match.arg(type)
  if (!file.exists(path)) stop("File does not exist: ", path, call. = FALSE)
  labels <- trimws(readLines(path, warn = FALSE, encoding = "UTF-8"))
  labels <- labels[nzchar(labels)]
  switch(
    type,
    character = labels,
    integer = as.integer(labels),
    numeric = as.numeric(labels),
    factor = factor(labels)
  )
}

test_that("sentences split at punctuation and lines", {
  result <- split_sentences("First sentence. Second one!\nThird?")
  expect_equal(result, c("First sentence.", "Second one!", "Third?"))
})

test_that("preprocessing applies requested transformations", {
  result <- preprocess_text(
    "The 2 QUICK, foxes!",
    stopwords = "the",
    min_token_length = 4
  )
  expect_equal(result, "quick foxes")
})

test_that("missing text becomes an empty document", {
  expect_equal(preprocess_text(c(NA, "Data")), c("", "data"))
})

test_that("label files support explicit types", {
  path <- tempfile(fileext = ".txt")
  writeLines(c("0", "1", "", "1"), path)
  expect_equal(read_label_file(path, "integer"), c(0L, 1L, 1L))
})

---
title: "Getting Started with Text Classification"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with Text Classification}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Overview

Text classification assigns documents to predefined categories. In
organizational research, documents might be vacancy sentences, employee
comments, reports, or interview excerpts. This tutorial develops a transparent
workflow from raw HTML to predictions.

```{r}
library(textclassificationtutorial)
```

## Extract text from HTML

The package includes the nursing-vacancy page used by the original tutorial.

```{r}
html_file <- system.file(
  "extdata", "sample_nursing_vacancy.html",
  package = "textclassificationtutorial"
)
vacancy_text <- extract_html_text(html_file)
substr(vacancy_text, 1, 200)
```

To process a folder, use `extract_html_dir()`. The result keeps a document ID,
the source path, and extracted text together.

```{r, eval = FALSE}
pages <- extract_html_dir("inst/extdata/vacancypages")
```

CSS and XPath selection are available when `xml2` is installed:

```{r, eval = FALSE}
extract_html_text(html_file, selector = "div.content")
extract_html_text(html_file, xpath = "//div[@class='content']")
```

## Segment and normalize

The unit of analysis should follow the research question. Here, each sentence
is treated as one document.

```{r}
sentences <- split_sentences(vacancy_text)
head(sentences)
```

Preprocessing choices are analytical decisions, not housekeeping. Removing
numbers may discard years of experience, and removing stopwords may discard
meaningful negation. Make each choice explicit.

```{r}
german_stopwords <- c(
  "der", "die", "das", "den", "dem", "des", "und", "oder", "mit",
  "für", "von", "zu", "im", "in", "auf", "ein", "eine"
)

clean <- preprocess_text(
  sentences,
  lowercase = TRUE,
  remove_punctuation = TRUE,
  remove_numbers = TRUE,
  stopwords = german_stopwords,
  min_token_length = 2
)
clean <- clean[nzchar(clean)]
head(clean)
```

## Create document features

```{r}
dtm <- document_term_matrix(
  clean,
  min_doc_freq = 2,
  max_doc_prop = 0.95
)
dtm
```

Rows represent documents, columns represent terms, and cells contain counts.
Use `binary = TRUE` when presence is more appropriate than frequency.

TF-IDF increases the weight of terms that are frequent in a particular
document but uncommon across the collection.

```{r}
weighted <- tf_idf(dtm)
keywords <- extract_keywords(dtm, n = 3)
head(keywords, 12)
```

## Explore similarity

Cosine similarity compares the orientation of two feature vectors while
reducing the influence of document length.

```{r}
similarity <- cosine_similarity(weighted)
round(similarity[1:min(5, nrow(similarity)),
                 1:min(5, ncol(similarity))], 2)
```

## Train a classifier

For a compact illustration, use synthetic documents with known labels.

```{r}
training_text <- c(
  "analyze data statistical model",
  "build predictive model data",
  "create dashboard analyze metrics",
  "provide nursing care patient",
  "support patient clinical care",
  "coordinate nurse patient treatment"
)
training_labels <- c("data", "data", "data", "care", "care", "care")

training_dtm <- document_term_matrix(training_text)
model <- fit_naive_bayes(training_dtm, training_labels, laplace = 1)
model

predicted <- predict(model, training_dtm)
classification_metrics(training_labels, predicted, positive = "data")
```

This training-set result demonstrates mechanics, not generalization. The next
vignette shows out-of-sample evaluation.

## Reproducible research checklist

- Preserve IDs and labels through every transformation.
- Define the unit of analysis before tokenization.
- Fit vocabulary and feature transformations on training data only.
- Prevent sentences from the same source document leaking across folds.
- Report preprocessing, class balance, tuning, and all evaluation metrics.
- Inspect errors and subgroup performance, not only aggregate accuracy.

---
title: "Evaluating a Text Classifier"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Evaluating a Text Classifier}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Why evaluation design matters

Text models can appear accurate because related documents occur in both
training and test data, preprocessing used the entire dataset, or accuracy
hides poor minority-class performance. Evaluation must mirror the intended
use of the classifier.

```{r}
library(textclassificationtutorial)

text <- c(
  "analyze customer data", "build statistical models",
  "create predictive analytics", "report business metrics",
  "provide patient care", "support clinical treatment",
  "coordinate nursing care", "assist hospital patients",
  "analyze experimental results", "develop data dashboard",
  "monitor patient health", "coordinate clinical team"
)
label <- rep(c("data", "care"), each = 4)
label <- c(label, "data", "data", "care", "care")
```

## Stratified folds

Stratification distributes each class across folds. The smallest class must
have at least `k` observations.

```{r}
folds <- stratified_folds(label, k = 3, seed = 2026)
folds
```

For nested documents—sentences within vacancies, employees within teams, or
posts within authors—split on the higher-level entity instead. Ordinary
stratification does not prevent grouped leakage.

## Cross-validation loop

The vocabulary must be learned from the training documents. Test documents are
then aligned to that training vocabulary by `predict.text_nb()`.

```{r}
fold_results <- lapply(folds, function(test_index) {
  train_index <- setdiff(seq_along(text), test_index)

  train_clean <- preprocess_text(text[train_index])
  test_clean <- preprocess_text(text[test_index])

  train_dtm <- document_term_matrix(train_clean)
  test_dtm <- document_term_matrix(test_clean)

  model <- fit_naive_bayes(train_dtm, label[train_index])
  estimate <- predict(model, test_dtm)

  classification_metrics(
    truth = label[test_index],
    estimate = estimate,
    positive = "data"
  )
})

results <- do.call(rbind, fold_results)
results
colMeans(results[c(
  "accuracy", "balanced_accuracy", "precision", "recall", "f1"
)], na.rm = TRUE)
```

## Interpret the metrics

- **Precision** asks what proportion of predicted positive documents is truly
  positive.
- **Recall** asks what proportion of true positive documents was found.
- **Specificity** is recall for the negative class.
- **Balanced accuracy** averages recall and specificity.
- **F1** is the harmonic mean of precision and recall.

Choose metrics before examining results. In rare-category classification,
ordinary accuracy can be high even when the classifier never detects the class
of interest.

## From tutorial to research

For a substantive study, add:

1. a held-out final test set;
2. grouped or temporal resampling where the data structure requires it;
3. training-fold-only tuning and feature selection;
4. confidence intervals across appropriate sampling units;
5. human review of false positives and false negatives;
6. documentation of annotation quality and inter-rater agreement;
7. fairness and drift checks for the deployment context.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtm.R
\name{document_term_matrix}
\alias{document_term_matrix}
\title{Construct a document-term matrix}
\usage{
document_term_matrix(
  text,
  document_ids = NULL,
  binary = FALSE,
  min_doc_freq = 1L,
  max_doc_prop = 1
)
}
\arguments{
\item{text}{Character vector containing one preprocessed document per item.}

\item{document_ids}{Optional unique document identifiers.}

\item{binary}{Logical; store term presence instead of term frequency?}

\item{min_doc_freq}{Minimum number of documents in which a term must occur.}

\item{max_doc_prop}{Maximum proportion of documents in which a term may
occur.}
}
\value{
A numeric matrix with class \code{text_dtm}.
}
\description{
Construct a document-term matrix
}
\examples{
docs <- preprocess_text(c("data science", "data analysis", "science"))
document_term_matrix(docs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess.R
\name{split_sentences}
\alias{split_sentences}
\title{Split text into sentences}
\usage{
split_sentences(
  text,
  split_lines = TRUE,
  keep_punctuation = TRUE,
  drop_empty = TRUE
)
}
\arguments{
\item{text}{Character vector.}

\item{split_lines}{Logical; treat line breaks and \code{|} as boundaries?}

\item{keep_punctuation}{Logical; retain terminal punctuation?}

\item{drop_empty}{Logical; remove empty results?}
}
\value{
A character vector of sentences.
}
\description{
A lightweight sentence segmenter suitable for tutorials and clean prose.
It splits at terminal punctuation followed by whitespace, and optionally at
line breaks and vertical bars. For production multilingual segmentation,
use a dedicated NLP tokenizer.
}
\examples{
split_sentences("Analyze data. Present results! Work with teams?")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/similarity.R
\name{cosine_similarity}
\alias{cosine_similarity}
\title{Compute cosine similarity}
\usage{
cosine_similarity(x, y = NULL)
}
\arguments{
\item{x}{Numeric matrix whose rows are observations.}

\item{y}{Optional numeric matrix with the same columns as \code{x}. When omitted,
computes all pairwise similarities among rows of \code{x}.}
}
\value{
A numeric similarity matrix. Similarities involving zero vectors
are returned as \code{NA}.
}
\description{
Compute cosine similarity
}
\examples{
x <- rbind(a = c(1, 1, 0), b = c(1, 0, 0), c = c(0, 0, 1))
cosine_similarity(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/naive_bayes.R
\name{fit_naive_bayes}
\alias{fit_naive_bayes}
\title{Fit a multinomial Naive Bayes text classifier}
\usage{
fit_naive_bayes(x, y, laplace = 1, prior = NULL)
}
\arguments{
\item{x}{Nonnegative numeric document-term matrix.}

\item{y}{Class labels with one value per row of \code{x}.}

\item{laplace}{Nonnegative additive smoothing parameter.}

\item{prior}{Optional named class probabilities.}
}
\value{
An object of class \code{text_nb}.
}
\description{
Fit a multinomial Naive Bayes text classifier
}
\examples{
x <- rbind(c(3, 0), c(2, 0), c(0, 3), c(0, 2))
colnames(x) <- c("analysis", "care")
model <- fit_naive_bayes(x, c("data", "data", "health", "health"))
predict(model, x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtm.R
\name{tf_idf}
\alias{tf_idf}
\title{Calculate TF-IDF weights}
\usage{
tf_idf(dtm, normalize = c("length", "max", "none"), smooth_idf = TRUE)
}
\arguments{
\item{dtm}{Numeric document-term matrix.}

\item{normalize}{Term-frequency normalization: document length, maximum
frequency, or none.}

\item{smooth_idf}{Logical; use smoothed inverse document frequency?}
}
\value{
A numeric matrix of TF-IDF weights.
}
\description{
Calculate TF-IDF weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{f_measure}
\alias{f_measure}
\title{F-measure}
\usage{
f_measure(precision, recall, beta = 1)
}
\arguments{
\item{precision}{Numeric precision.}

\item{recall}{Numeric recall.}

\item{beta}{Relative weight assigned to recall.}
}
\value{
Numeric F-measure.
}
\description{
F-measure
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/html.R
\name{extract_html_dir}
\alias{extract_html_dir}
\title{Extract text from a directory of HTML files}
\usage{
extract_html_dir(path, pattern = "\\\\.html?$", recursive = FALSE, ...)
}
\arguments{
\item{path}{Directory containing HTML files.}

\item{pattern}{File-name regular expression.}

\item{recursive}{Logical; search recursively?}

\item{...}{Passed to \code{\link[=extract_html_text]{extract_html_text()}}.}
}
\value{
A data frame with \code{document_id}, \code{path}, and \code{text}.
}
\description{
Extract text from a directory of HTML files
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/html.R
\name{extract_html_text}
\alias{extract_html_text}
\title{Extract readable text from HTML}
\usage{
extract_html_text(
  x,
  selector = NULL,
  xpath = NULL,
  collapse = "\\n",
  trim = TRUE
)
}
\arguments{
\item{x}{Path to an HTML file or a length-one HTML character string.}

\item{selector}{Optional CSS selector.}

\item{xpath}{Optional XPath expression. Supply at most one of \code{selector}
and \code{xpath}.}

\item{collapse}{Character used to join matched nodes.}

\item{trim}{Logical; normalize whitespace and trim the result?}
}
\value{
A length-one character vector containing extracted text.
}
\description{
Extracts text from an HTML file or character string. When the suggested
\code{xml2} package is installed, simple CSS selectors (\code{tag}, \code{.class}, \verb{#id},
or \code{tag.class}) or XPath can target a specific part of the page. A
dependency-free fallback strips markup from the full document.
}
\examples{
html <- "<html><body><h1>Analyst</h1><p>Analyze data.</p></body></html>"
extract_html_text(html)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess.R
\name{preprocess_text}
\alias{preprocess_text}
\title{Normalize text for document-term analysis}
\usage{
preprocess_text(
  text,
  lowercase = TRUE,
  remove_punctuation = TRUE,
  remove_numbers = TRUE,
  stopwords = character(),
  min_token_length = 1L
)
}
\arguments{
\item{text}{Character vector.}

\item{lowercase}{Logical; convert text to lowercase?}

\item{remove_punctuation}{Logical; replace punctuation with spaces?}

\item{remove_numbers}{Logical; replace digits with spaces?}

\item{stopwords}{Optional character vector of words to remove.}

\item{min_token_length}{Minimum number of characters per token.}
}
\value{
A character vector of normalized documents.
}
\description{
Normalize text for document-term analysis
}
\examples{
preprocess_text(
  c("Analyze the data!", "Present 2 reports."),
  stopwords = c("the")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess.R
\name{read_label_file}
\alias{read_label_file}
\title{Read a one-label-per-line file}
\usage{
read_label_file(path, type = c("character", "integer", "numeric", "factor"))
}
\arguments{
\item{path}{Path to a text file.}

\item{type}{Return labels as character, integer, numeric, or factor.}
}
\value{
A vector of labels.
}
\description{
Read a one-label-per-line file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{classification_metrics}
\alias{classification_metrics}
\title{Calculate binary classification metrics}
\usage{
classification_metrics(truth, estimate, positive)
}
\arguments{
\item{truth}{Vector of true classes.}

\item{estimate}{Vector of predicted classes.}

\item{positive}{Value identifying the positive class.}
}
\value{
A one-row data frame containing confusion counts, accuracy,
balanced accuracy, precision, recall, specificity, and F1.
}
\description{
Calculate binary classification metrics
}
\examples{
classification_metrics(c("task", "task", "other"), c("task", "other", "other"),
                       positive = "task")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluation.R
\name{stratified_folds}
\alias{stratified_folds}
\title{Create stratified cross-validation folds}
\usage{
stratified_folds(y, k = 5L, repeats = 1L, seed = NULL)
}
\arguments{
\item{y}{Class labels.}

\item{k}{Number of folds.}

\item{repeats}{Number of repeated fold sets.}

\item{seed}{Optional random seed. The caller's random-number state is
restored.}
}
\value{
A list of integer test-set indices with class \code{text_folds}.
}
\description{
Create stratified cross-validation folds
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dtm.R
\name{extract_keywords}
\alias{extract_keywords}
\title{Extract top TF-IDF keywords}
\usage{
extract_keywords(x, n = 1L, already_tfidf = FALSE)
}
\arguments{
\item{x}{A document-term matrix or TF-IDF matrix.}

\item{n}{Number of keywords per document.}

\item{already_tfidf}{Logical; is \code{x} already weighted?}
}
\value{
A data frame with document, rank, term, and weight.
}
\description{
Extract top TF-IDF keywords
}
